/**
 * Copyright (C) 2009-2013 Couchbase, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALING
 * IN THE SOFTWARE.
 */

package com.couchbase.client.protocol.views;

import java.util.Collection;
import java.util.Map;

/**
 * Holds the response of a view query where the map and reduce
 * function were called.
 */
public class ViewResponseReduced extends ViewResponse {

  public ViewResponseReduced(final Collection<ViewRow> rows,
      final Collection<RowError> errors) {
    super(rows, errors, -1);
  }

  public ViewResponseReduced(final Collection<ViewRow> rows,
      final Collection<RowError> errors, long totalViewRows) {
    super(rows, errors, totalViewRows);
  }

  @Override
  public Map<String, Object> getMap() {
    throw new UnsupportedOperationException("This view doesn't contain "
        + "documents");
  }

  @Override
  public String toString() {
    StringBuilder s = new StringBuilder();
    for (ViewRow r : rows) {
      s.append(r.getKey());
      s.append(" : ");
      s.append(r.getValue());
      s.append("\n");
    }
    return s.toString();
  }

  @Override
  public long getTotalRows() {
    throw new IllegalStateException("Total Number of Rows is not available on "
      + "reduced views.");
  }

}
